USE AdventureWorks
GO
-- Prozedur fr Webservice erstellen
IF OBJECT_ID ( 'Production.GetProductInfo', 'P' ) IS NOT NULL 
    DROP PROCEDURE Production.GetProductInfo
GO
CREATE PROCEDURE Production.GetProductInfo ( 
 @ProductNumber nvarchar(25)
) AS BEGIN
SELECT ProductNumber AS "@P-Nr", 
       Name          AS "Name", 
       StandardCost  AS "Prices/Standard",
       ListPrice     AS "Prices/List",
       Size          AS "Details/Size",
       Color         AS "Details/Color"
  FROM Production.Product
 WHERE ProductNumber = @ProductNumber
 ORDER BY ProductID DESC
FOR XML PATH('Product'), ROOT('Product-List')
END
GO
-- Funktion fr Webservice erstellen
IF OBJECT_ID ( 'Production.GetProductsPerSubcategory', 'FN' ) IS NOT NULL 
    DROP FUNCTION Production.GetProductsPerSubcategory
GO
CREATE FUNCTION Production.GetProductsPerSubcategory(
  @vProductSubcategoryID int
) 
RETURNS int
AS BEGIN
 DECLARE @vProducts int
 SELECT @vProducts = COUNT(*)
   FROM Production.Product
  WHERE ProductSubcategoryID = @vProductSubcategoryID
 RETURN @vProducts
END
GO
-- Endpunkt lschen
DROP ENDPOINT epProduct
GO
-- Namensraum lschen und anlegen
EXEC sp_delete_http_namespace_reservation N'http://localhost:80/sql'
EXEC sp_reserve_http_namespace N'http://localhost:80/sql' 
GO
-- Webservice mit Prozedur und Funktion anlegen
CREATE ENDPOINT epProduct 
   STATE = STARTED
AS HTTP(
   PATH = '/sql', 
   AUTHENTICATION = (INTEGRATED ), 
   PORTS = ( CLEAR ),
   SITE = 'localhost'
)
FOR SOAP (
   WEBMETHOD 'http://tempUri.org/'.'GetProductInfo' 
            (name='AdventureWorks.Production.GetProductInfo', 
             schema=STANDARD ),
   WEBMETHOD 'http://tempUri.org/'.'GetProductsPerSubcategory' 
            (name='AdventureWorks.Production.GetProductsPerSubcategory', 
             schema=STANDARD ),
   BATCHES = DISABLED,
   WSDL = DEFAULT,
   DATABASE = 'AdventureWorks',
   NAMESPACE = 'http://Adventure-Works/Products'
)
GO
USE master
EXEC sp_grantlogin @loginame='NOTEBOOK\Comelio'
EXEC sp_grantdbaccess @loginame='NOTEBOOK\Comelio'
GRANT CONNECT ON ENDPOINT::epProduct TO [NOTEBOOK\Comelio]
GO