USE AdventureWorks
GO
-- Funktion fr die Ermittlung der Menge von Produkten auf dem Lager
IF OBJECT_ID('Sales.usfGetQuantity', 'FN') IS NOT NULL
 DROP FUNCTION Sales.usfGetQuantity 
GO
CREATE FUNCTION Sales.usfGetQuantity(
  @productNumber nvarchar(25)
) RETURNS int
AS BEGIN
-- Variable fr Mengenspeicherung
DECLARE @quantity int
-- Ermittlung der verfgbaren Menge
SELECT @quantity = SUM(Quantity)
  FROM Production.ProductInventory AS piv 
       INNER JOIN Production.Product AS p
       ON p.ProductID = piv.ProductID
       INNER JOIN Production.Location AS l
       ON piv.LocationID = l.LocationID
WHERE ProductNumber = @productNumber
GROUP BY ProductNumber
-- Rckgabe
RETURN @quantity
END
-- Test der Funktion
/*
SELECT Sales.usfGetQuantity('BA-8327')
*/
-- XML Schema fr ProductOrder anmelden
IF ((SELECT COUNT(*)  
      FROM sys.xml_schema_collections 
     WHERE [name] = 'ProductOrder') > 0) BEGIN
 DROP XML SCHEMA COLLECTION Sales.ProductOrder 
END
GO
CREATE XML SCHEMA COLLECTION Sales.ProductOrder AS
N'<?xml version="1.0"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema" elementFormDefault="qualified" attributeFormDefault="unqualified" xmlns:c="http://www.contoso.com/ProductOrder" targetNamespace="http://www.contoso.com/ProductOrder">
  <xs:element name="ProductOrder">
    <xs:complexType>
      <xs:sequence>
        <xs:element name="ProductNumber" type="xs:string"/>
        <xs:element name="Quantity" type="xs:int"/>
        <xs:element name="AccountNumber" type="xs:string"/>
      </xs:sequence>
    </xs:complexType>
  </xs:element>
</xs:schema>'
GO

-- XML Schema fr ProductOrderConfirmation anmelden
IF ((SELECT COUNT(*)  
      FROM sys.xml_schema_collections 
     WHERE [name] = 'ProductOrderConfirmation') > 0) BEGIN
 DROP XML SCHEMA COLLECTION Sales.ProductOrderConfirmation 
END
CREATE XML SCHEMA COLLECTION Sales.ProductOrderConfirmation AS
N'<?xml version="1.0"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema" elementFormDefault="qualified" attributeFormDefault="unqualified" xmlns:c="http://www.contoso.com/ProductOrderConfirmation" targetNamespace="http://www.contoso.com/ProductOrderConfirmation">
  <xs:element name="ProductOrderConfirmation">
    <xs:complexType>
      <xs:choice>
        <xs:element name="Confirmation">
          <xs:complexType>
            <xs:sequence>
              <xs:element name="ProductNumber" type="xs:string"/>
              <xs:element name="Quantity" type="xs:int"/>
              <xs:element name="ShipDate" type="xs:string"/>
              <xs:element name="AccountNumber" type="xs:string"/>
            </xs:sequence>
          </xs:complexType>
        </xs:element>
        <xs:element name="Cancellation">
          <xs:complexType>
            <xs:sequence>
              <xs:element name="Comment" type="xs:string"/>
            </xs:sequence>
          </xs:complexType>
        </xs:element>
      </xs:choice>
    </xs:complexType>
  </xs:element>
</xs:schema>'
GO

-- Prozedur fr Webservice erstellen
IF OBJECT_ID ( 'Production.ProductOrder', 'FN' ) IS NOT NULL 
    DROP FUNCTION Production.ProductOrder
GO
CREATE FUNCTION Production.ProductOrder ( 
 @order xml(Sales.ProductOrder) 
)
RETURNS xml(Sales.ProductOrderConfirmation)
AS BEGIN
-- XML-Spaltenvariablen, Hilfsvariablen
DECLARE @productNumber varchar(50),
        @quantity int,
        @accountNumber varchar(25),
        @availableProducts int,
        @confirmation xml(Sales.ProductOrderConfirmation)
-- Zerlegung
SET @productNumber = @order.value('declare namespace c="http://www.contoso.com/ProductOrder"; (//c:ProductNumber)[1]', 'varchar(25)')
SET @quantity = @order.value('declare namespace c="http://www.contoso.com/ProductOrder"; (//c:Quantity)[1]', 'int')
SET @accountNumber = @order.value('declare namespace c="http://www.contoso.com/ProductOrder"; (//c:AccountNumber)[1]', 'varchar(25)')
-- Erhltlichkeit prfen?
SELECT @availableProducts = Sales.usfGetQuantity(@productNumber)
-- Produkt erhltlich
IF  @availableProducts > 0 BEGIN
 -- Bestellung auslsen und Prozesse anstoen...
 SET @confirmation = N'<c:ProductOrderConfirmation xmlns:c="http://www.contoso.com/ProductOrderConfirmation">
  <c:Confirmation>
    <c:ProductNumber>BA-8327</c:ProductNumber>
    <c:Quantity>3</c:Quantity>
    <c:ShipDate></c:ShipDate>
    <c:AccountNumber>AW00019908</c:AccountNumber>
  </c:Confirmation>
</c:ProductOrderConfirmation>'
END 
-- Produkt nicht erhltlich
ELSE
 SET @confirmation =  N'<c:ProductOrderConfirmation xmlns:c="http://www.contoso.com/ProductOrderConfirmation">
  <c:Cancellation>
    <c:Comment>Produkt nicht verfgbar.</c:Comment>
  </c:Cancellation>
</c:ProductOrderConfirmation>'
RETURN @confirmation
END 
GO

-- Test der Funktion
/*
DECLARE @order xml(Sales.ProductOrder),
        @confirmation xml(Sales.ProductOrderConfirmation)
SET @order = '<c:ProductOrder xmlns:c="http://www.contoso.com/ProductOrder">
  <c:ProductNumber>BA-8327</c:ProductNumber>
  <c:Quantity>3</c:Quantity>
  <c:AccountNumber>AW00019908</c:AccountNumber>
</c:ProductOrder>'
SET @confirmation = Production.ProductOrder(@order)
PRINT CAST(@confirmation AS varchar(max))
*/

-- Endpunkt lschen
DROP ENDPOINT epProduct
GO
-- Namensraum lschen und anlegen
EXEC sp_delete_http_namespace_reservation N'http://localhost:80/sql'
EXEC sp_reserve_http_namespace N'http://localhost:80/sql' 
GO
-- Webservice mit Prozedur und Funktion anlegen
CREATE ENDPOINT epProduct 
   STATE = STARTED
AS HTTP(
   PATH = '/sql', 
   AUTHENTICATION = (INTEGRATED ), 
   PORTS = ( CLEAR ),
   SITE = 'localhost'
)
FOR SOAP (
   WEBMETHOD 'http://tempUri.org/'.'ProductOrder' 
            (name='AdventureWorks.Production.ProductOrder', 
             schema=STANDARD ),
   BATCHES = DISABLED,
   WSDL = DEFAULT,
   DATABASE = 'AdventureWorks',
   NAMESPACE = 'http://Adventure-Works/Products'
)
GO
USE master
EXEC sp_grantlogin @loginame='NOTEBOOK\Comelio'
EXEC sp_grantdbaccess @loginame='NOTEBOOK\Comelio'
GRANT CONNECT ON ENDPOINT::epProduct TO [NOTEBOOK\Comelio]
GO