DROP PROCEDURE Production.CustomWSDL
GO
CREATE PROCEDURE Production.CustomWSDL
    @endpointID int,
    @IsSSL bit,
    @Host nvarchar(128),
    @QueryString nvarchar(128),
    @UserAgent nvarchar(128)
as
BEGIN
  SET nocount on
  DECLARE @http int, @soap int, @started int, @outputWSDL nvarchar(max)
  SET @http = 1
  SET @soap = 1
  SET @started = 0
  -- Einfache Zeichenketten vorbereiten, die spter zusammengesetzt werden knnen
  SET @outputWSDL = N'<wsdl:definitions xmlns:wsdl="http://schemas.xmlsoap.org/wsdl/" xmlns:soap="http://schemas.xmlsoap.org/wsdl/soap/"'
  DECLARE @wsdlStartTypes nvarchar(50)
  SET @wsdlStartTypes = N'<wsdl:types>'
  DECLARE @wsdlEndTypes nvarchar(20)
  SET @wsdlEndTypes = N'</wsdl:types>'
  DECLARE @xsdStartSchema nvarchar(150)
  SET @xsdStartSchema = N'<xsd:schema xmlns:xsd="http://www.w3.org/2001/XMLSchema" attributeFormDefault="qualified" elementFormDefault="qualified" targetNamespace='
  DECLARE @xsdEndSchema nvarchar(20)
  SET @xsdEndSchema = N'</xsd:schema>'
  DECLARE @xsdStartElement nvarchar (50)
  SET @xsdStartElement = N'<xsd:element name='
  DECLARE @xsdEndElement nvarchar(20)
  SET @xsdEndElement = N'</xsd:element>'
  DECLARE @xsdStartComplexType nvarchar(50)
  SET @xsdStartComplexType = N'<xsd:complexType><xsd:sequence>'
  DECLARE @xsdEndComplexType nvarchar(50)
  SET @xsdEndComplexType = N'</xsd:sequence></xsd:complexType>'
  DECLARE @wsdlStartMessage nvarchar(100)
  SET @wsdlStartMessage = N'<wsdl:message name='
  DECLARE @wsdlEndMessage nvarchar(20)
  SET @wsdlEndMessage = N'</wsdl:message>'
  DECLARE @wsdlStartPart nvarchar(50)
  SET @wsdlStartPart = N'<wsdl:part name="parameters" element='
  DECLARE @wsdlEndPart nvarchar(20)
  SET @wsdlEndPart = N'</wsdl:part>'
  DECLARE @wsdlStartPortType nvarchar(50)
  SET @wsdlStartPortType = N'<wsdl:portType name='
  DECLARE @wsdlEndPortType nvarchar(20)
  SET @wsdlEndPortType = N'</wsdl:portType>'
  DECLARE @wsdlStartOperation nvarchar(50)
  SET @wsdlStartOperation = N'<wsdl:operation name='
  DECLARE @wsdlEndOperation nvarchar(20)
  SET @wsdlEndOperation = N'</wsdl:operation>'
  DECLARE @wsdlStartInput nvarchar(50)
  SET @wsdlStartInput = N'<wsdl:input name='
  DECLARE @wsdlEndInput nvarchar(20)
  SET @wsdlEndInput = N'</wsdl:input>'
  DECLARE @wsdlStartOutput nvarchar(50)
  SET @wsdlStartOutput = N'<wsdl:output name='
  DECLARE @wsdlEndOutput nvarchar(20)
  SET @wsdlEndOutput = N'</wsdl:output>'
  DECLARE @wsdlStartBinding nvarchar(50)
  SET @wsdlStartBinding = N'<wsdl:binding name='
  DECLARE @wsdlEndBinding nvarchar(20)
  SET @wsdlEndBinding = N'</wsdl:binding>'
  DECLARE @soapBinding nvarchar(100)
  SET @soapBinding = N'<soap:binding transport="http://schemas.xmlsoap.org/soap/http" style="document"/>'
  DECLARE @soapStartOperation nvarchar(50)
  SET @soapStartOperation = N'<soap:operation soapAction='
  DECLARE @soapEndOperation nvarchar(20)
  SET @soapEndOperation = N' style="document" />'
  DECLARE @soapBody nvarchar(50)
  SET @soapBody = N'<soap:body use="literal" />'
  DECLARE @wsdlStartService nvarchar(50)
  SET @wsdlStartService = N'<wsdl:service name='
  DECLARE @wsdlEndService nvarchar(20)
  SET @wsdlEndService = N'</wsdl:service>'
  DECLARE @wsdlStartPort nvarchar(50)
  SET @wsdlStartPort = N'<wsdl:port name='
  DECLARE @wsdlEndPort nvarchar(20)
  SET @wsdlEndPort = N'</wsdl:port>'
  DECLARE @soapStartAddress nvarchar(50)
  SET @soapStartAddress = N'<soap:address location='
  DECLARE @soapEndAddress nvarchar(20)
  SET @soapEndAddress = N'</soap:address>'
  DECLARE @wsdlEndDefinitions nvarchar(20)
  SET @wsdlEndDefinitions = N'</wsdl:definitions>'
  
  -- Weitere Variablen
  DECLARE @endpointWsdl nvarchar(100),
          @endpointProtocol int,
          @endpointType int,
          @endpointState int,
          @endpointBatches bit,
          @endpointMethodCount int

  -- Ist WSDL eingeschaltet?
  SELECT @endpointWsdl = wsdl_generator_procedure
   FROM sys.soap_endpoints
  WHERE endpoint_id = @endpointID

  IF (NOT (LEN(@endpointWsdl) > 0))
  BEGIN
    raiserror ('WSDL-Generierung ist ausgeschaltet.', 16, 1)
  END

  -- Wird nach WSDL gefragt?
  IF (N'WSDL' <> UPPER(@QueryString))
  BEGIN
    raiserror ('Anfrage nicht untersttzt. WSDL verlangt.', 16, 1)
  END

  -- Existiert der Endpunkt?
  IF ((SELECT endpoint_id FROM sys.endpoints WHERE endpoint_id = @endpointID) is NULL)
  BEGIN
     raiserror ('Endpunkt nicht vorhanden.', 16, 1)
  END

  -- berprfen, ob ein SOAP-Endpunkt nachgefragt wird und ob er gestartet ist
  SELECT @endpointProtocol=protocol,
         @endpointType=type,
         @endpointState=state
    FROM sys.endpoints
   WHERE endpoint_id = @endpointID
 
  IF ((@endpointProtocol <> @http) OR (@endpointType <> @soap) OR (@endpointState <> @started))
  BEGIN
    RAISERROR('Endpunkt ist nicht gestartet oder kein SOAP-Endpunkt.', 16, 1)
  END

  --  Ist SqlBatch eingeschaltet?
  SELECT @endpointBatches = is_sql_language_enabled
    FROM sys.soap_endpoints
   WHERE endpoint_id = @endpointID

  -- Wieviele Methoden sind vorhanden?
  SELECT @endpointMethodCount = count(*)
    FROM sys.endpoint_webmethods
   WHERE endpoint_id = @endpointID

  -- Wenn SqlBatch oder eine Operation vorhanden, dann kann WSDL erstellt werden
  IF ((@endpointBatches = 1) OR (@endpointMethodCount > 0))
  BEGIN
    -- Hier fehlt noch die Erzeugung von sqlBatch-WSDL
    CREATE TABLE #tempWSDLMethod (db nvarchar(20) NOT NULL, 
                                  oOwner nvarchar(20) NOT NULL, 
                                  oName nvarchar(50) NOT NULL)

    INSERT #tempWSDLMethod (db, oOwner, oName)
    SELECT LEFT(object_name, CHARINDEX(N'.', object_name)-1), 
           SUBSTRING(object_name, CHARINDEX(N'.', object_name)+1, CHARINDEX(N'.', object_name, CHARINDEX(N'.', object_name)+1)-CHARINDEX(N'.', object_name)-1),
           RIGHT(object_name, LEN(object_name)-CHARINDEX(N'.', object_name, CHARINDEX(N'.', object_name)+1))
      FROM sys.endpoint_webmethods
     WHERE endpoint_id = @endpointID

    CREATE TABLE #tempWSDLTABLE (id int identity primary key, 
                                 webMethodNamespace nvarchar(max) NOT NULL, 
                                 webMethodName nvarchar(max) NOT NULL,
                                 paramName nvarchar(100) NOT NULL,
                                 namespaceSuffix int DEFAULT 1)

    DECLARE webMethodDb_Cursor CURSOR FOR
     SELECT DISTINCT db
       FROM #tempWSDLMethod

    DECLARE @tDbName nvarchar(50)

    OPEN webMethodDb_Cursor
    FETCH NEXT FROM webMethodDb_Cursor INTO @tDbName
    WHILE @@FETCH_STATUS = 0
    BEGIN
        DECLARE @tQuery nvarchar(max)
        SET @tQuery = N'use ' + @tDbName + '; INSERT #tempWSDLTABLE (webMethodNamespace, webMethodName, paramName)
            SELECT a.namespace, a.method_alias, RIGHT(b.name, (LEN(b.name)-1))
            FROM sys.endpoint_webmethods AS a, sys.parameters AS b
            WHERE a.endpoint_id = ' + CAST(@endpointID AS nvarchar(10))
            + N' and b.object_id = object_id(a.object_name) and (LEN(b.name) > 0)' 
        EXEC (@tQuery)
        FETCH NEXT FROM webMethodDb_Cursor INTO @tDbName
    END

    CLOSE webMethodDb_Cursor
    DEALLOCATE webMethodDb_Cursor

    -- WSDL erstellen
    SELECT @outputWSDL = @outputWSDL + N' xmlns:tns="' + default_namespace + 
          '" targetNamespace="' + default_namespace + '"'
      FROM sys.soap_endpoints
     WHERE endpoint_id = @endpointID

    DECLARE webMethodNS_Cursor SCROLL CURSOR FOR
     SELECT DISTINCT webMethodNamespace FROM #tempWSDLTABLE

    DECLARE @tCount int, 
            @tMethodNS varchar(50)
    SET @tCount = 1

    OPEN webMethodNS_Cursor
    FETCH NEXT FROM webMethodNS_Cursor INTO @tMethodNS
    WHILE @@FETCH_STATUS = 0
    BEGIN
        SELECT @outputWSDL = @outputWSDL + N' xmlns:s' + CAST(@tCount AS nvarchar(3)) + N'="' + @tMethodNS + N'"'
        UPDATE #tempWSDLTABLE SET namespaceSuffix = @tCount WHERE webMethodNamespace = @tMethodNS
        SET @tCount = @tCount + 1
        FETCH NEXT FROM webMethodNS_Cursor INTO @tMethodNS
    END

    SELECT @outputWSDL = @outputWSDL + N'>'

    -- <wsdl:types>
    SELECT @outputWSDL = @outputWSDL + @wsdlStartTypes
    -- XML Schema
    FETCH FIRST FROM webMethodNS_Cursor INTO @tMethodNS
    WHILE @@FETCH_STATUS = 0
    BEGIN
        SELECT @outputWSDL = @outputWSDL + @xsdStartSchema + N'"' + @tMethodNS + N'">'

        DECLARE webMethodInfo_Cursor CURSOR FOR
        SELECT DISTINCT webMethodName, paramName 
            FROM #tempWSDLTABLE 
            WHERE webMethodNamespace = @tMethodNS
            order by webMethodName ASC

        DECLARE @tMethodName nvarchar(50),
                @tMethodNameBak nvarchar(50),
                @tMethodParamName nvarchar(20),
                @bFirstTime bit
        SET @tMethodNameBak = N''
        SET @bFirstTime = 1
        OPEN webMethodInfo_Cursor
        FETCH NEXT FROM webMethodInfo_Cursor INTO @tMethodName, @tMethodParamName
        WHILE @@FETCH_STATUS = 0
        BEGIN
           IF ((NOT (@bFirstTime = 1) AND (@tMethodNameBak <> @tMethodName)))
           BEGIN
               
               SELECT @outputWSDL = @outputWSDL + @xsdEndComplexType
               SELECT @outputWSDL = @outputWSDL + @xsdEndElement

               
               SELECT @outputWSDL = @outputWSDL + @xsdStartElement + N'"' + @tMethodNameBak + N'Response">'
               SELECT @outputWSDL = @outputWSDL + @xsdStartComplexType
               SELECT @outputWSDL = @outputWSDL + @xsdStartElement + N'"' + @tMethodNameBak + N'Result">'
               SELECT @outputWSDL = @outputWSDL + @xsdEndElement
               SELECT @outputWSDL = @outputWSDL + @xsdEndComplexType
               SELECT @outputWSDL = @outputWSDL + @xsdEndElement
           END

           -- Nachrichtenstruktur
           IF (@tMethodNameBak <> @tMethodName)
           BEGIN
              -- Methodenname
              SELECT @outputWSDL = @outputWSDL + @xsdStartElement + N'"' + @tMethodName + N'">'
              SELECT @outputWSDL = @outputWSDL + @xsdStartComplexType
           END
           
           -- Parameter
           SELECT @outputWSDL = @outputWSDL + @xsdStartElement + N'"' + @tMethodParamName + N'">'
           SELECT @outputWSDL = @outputWSDL + @xsdEndElement

           SET @bFirstTime = 0
           SET @tMethodNameBak = @tMethodName
           FETCH NEXT FROM webMethodInfo_Cursor INTO @tMethodName, @tMethodParamName
        END

        SELECT @outputWSDL = @outputWSDL + @xsdEndComplexType
        SELECT @outputWSDL = @outputWSDL + @xsdEndElement

        -- Antwort-Nachricht
        SELECT @outputWSDL = @outputWSDL + @xsdStartElement + N'"' + @tMethodNameBak + N'Response">'
        SELECT @outputWSDL = @outputWSDL + @xsdStartComplexType
        SELECT @outputWSDL = @outputWSDL + @xsdStartElement + N'"' + @tMethodNameBak + N'Result">'
        SELECT @outputWSDL = @outputWSDL + @xsdEndElement
        SELECT @outputWSDL = @outputWSDL + @xsdEndComplexType
        SELECT @outputWSDL = @outputWSDL + @xsdEndElement

        CLOSE webMethodInfo_Cursor
        DEALLOCATE webMethodInfo_Cursor

        SELECT @outputWSDL = @outputWSDL + @xsdEndSchema

        FETCH NEXT FROM webMethodNS_Cursor INTO @tMethodNS
     END
        
    CLOSE webMethodNS_Cursor
    DEALLOCATE webMethodNS_Cursor
    
    -- </wsdl:types>
    SELECT @outputWSDL = @outputWSDL + @wsdlEndTypes

    -- Traversieren der Operationen
    DECLARE webMethodInfo_Cursor SCROLL CURSOR FOR
        SELECT DISTINCT webMethodName, webMethodNamespace, namespaceSuffix 
        FROM #tempWSDLTABLE
        ORDER BY webMethodNamespace ASC, webMethodName ASC

    DECLARE @tIdSuffix int
    
    OPEN webMethodInfo_Cursor
    FETCH FIRST FROM webMethodInfo_Cursor INTO @tMethodName, @tMethodNS, @tIdSuffix
    WHILE @@FETCH_STATUS = 0
    BEGIN
        -- <wsdl:message>
        SELECT @outputWSDL = @outputWSDL + @wsdlStartMessage + N'"s' + CAST(@tIdSuffix AS nvarchar(3)) + N'Msg' + @tMethodName + N'SoapIn">'
        SELECT @outputWSDL = @outputWSDL + @wsdlStartPart + N'"s' + CAST(@tIdSuffix AS nvarchar(3)) + N':' + @tMethodName + N'" />'
        SELECT @outputWSDL = @outputWSDL + @wsdlEndMessage
        SELECT @outputWSDL = @outputWSDL + @wsdlStartMessage + N'"s' + CAST(@tIdSuffix AS nvarchar(3)) + N'Msg' + @tMethodName + N'SoapOut">'
        SELECT @outputWSDL = @outputWSDL + @wsdlStartPart + N'"s' + CAST(@tIdSuffix AS nvarchar(3)) + N':' + @tMethodName + N'Response" />'
        SELECT @outputWSDL = @outputWSDL + @wsdlEndMessage
        FETCH Next FROM webMethodInfo_Cursor INTO @tMethodName, @tMethodNS, @tIdSuffix
    END

    -- <wsdl:portType>
    SELECT @outputWSDL = @outputWSDL + @wsdlStartPortType + N'"' + name + N'Soap">'
        FROM sys.http_endpoints WHERE endpoint_id = @endpointID

    FETCH FIRST FROM webMethodInfo_Cursor INTO @tMethodName, @tMethodNS, @tIdSuffix
    WHILE @@FETCH_STATUS = 0
    BEGIN
        -- <wsdl:Operation>
        SELECT @outputWSDL = @outputWSDL + @wsdlStartOperation + N'"' + @tMethodName + N'">'
        SELECT @outputWSDL = @outputWSDL + @wsdlStartInput + N'"s' + CAST(@tIdSuffix AS nvarchar(3)) + N'Msg' + @tMethodName + 
                             N'SoapIn" message="tns:s' + CAST(@tIdSuffix AS nvarchar(3)) + N'Msg' + @tMethodName + N'SoapIn" />'
        SELECT @outputWSDL = @outputWSDL + @wsdlStartOutput + N'"s' + CAST(@tIdSuffix AS nvarchar(3)) + N'Msg' + @tMethodName +
                             N'SoapOut" message="tns:s' + CAST(@tIdSuffix AS nvarchar(3)) + N'Msg' + @tMethodName + N'SoapOut" />'
        SELECT @outputWSDL = @outputWSDL + @wsdlEndOperation
        FETCH NEXT FROM webMethodInfo_Cursor INTO @tMethodName, @tMethodNS, @tIdSuffix
    END

    SELECT @outputWSDL = @outputWSDL + @wsdlEndPortType

    -- <wsdl:binding>
    SELECT @outputWSDL = @outputWSDL + @wsdlStartBinding + N'"' + name + N'Soap" type="tns:' + name + N'Soap">'
      FROM sys.http_endpoints
     WHERE endpoint_id = @endpointID

    SELECT @outputWSDL = @outputWSDL + @soapBinding

    FETCH FIRST FROM webMethodInfo_Cursor INTO @tMethodName, @tMethodNS, @tIdSuffix
    WHILE @@FETCH_STATUS = 0
    BEGIN
        SELECT @outputWSDL = @outputWSDL + @wsdlStartOperation + N'"' + @tMethodName + N'">'
        SELECT @outputWSDL = @outputWSDL + @soapStartOperation + N'"' + @tMethodNS + @tMethodName + N'"' + @soapEndOperation
        SELECT @outputWSDL = @outputWSDL + @wsdlStartInput + N'"s' + CAST(@tIdSuffix AS nvarchar(3)) + N'Msg' + @tMethodName + N'SoapIn">'
        SELECT @outputWSDL = @outputWSDL + @soapBody + @wsdlEndInput
        SELECT @outputWSDL = @outputWSDL + @wsdlStartOutput + N'"s' + CAST(@tIdSuffix AS nvarchar(3)) + N'Msg' + @tMethodName + N'SoapOut">'
        SELECT @outputWSDL = @outputWSDL + @soapBody + @wsdlEndOutput + @wsdlEndOperation
        FETCH NEXT FROM webMethodInfo_Cursor INTO @tMethodName, @tMethodNS, @tIdSuffix
    END

    CLOSE webMethodInfo_Cursor
    DEALLOCATE webMethodInfo_Cursor

    SELECT @outputWSDL = @outputWSDL + @wsdlEndBinding

    SELECT @outputWSDL = @outputWSDL + @wsdlStartService + N'"' + name + N'">'
      FROM sys.http_endpoints 
     WHERE endpoint_id = @endpointID

    SELECT @outputWSDL = @outputWSDL + @wsdlStartPort + N'"' + name + N'" binding="tns:' + name + N'Soap">'
      FROM sys.http_endpoints
     WHERE endpoint_id = @endpointID

    IF (@IsSSL = 1)
    BEGIN
        SELECT @outputWSDL = @outputWSDL + @soapStartAddress + N'"https://' + @Host + N'/' + url_path + N'" />'
          FROM sys.http_endpoints
         WHERE endpoint_id = @endpointID
    END
    else
    BEGIN
        SELECT @outputWSDL = @outputWSDL + @soapStartAddress + N'"http://' + @Host + N'/' + url_path + N'" />'
          FROM sys.http_endpoints WHERE endpoint_id = @endpointID
    END    

    SELECT @outputWSDL = @outputWSDL + @wsdlEndPort + @wsdlEndService + @wsdlEndDefinitions
  END

  -- The WSDL document must be returned to the client using this GUID AS the column name.
  SELECT @outputWSDL AS N'XML_F52E2B61-18A1-11d1-B105-00805F49916B'
END
GO

