USE AdventureWorks
GO
/*
    Datenstrukturen
*/
-- Tabelle erstellen
IF OBJECT_ID ('Production.Product2') IS NOT NULL
DROP TABLE Production.Product2
GO
CREATE TABLE Production.Product2 (
 ProductID     int IDENTITY(1,1) NOT NULL PRIMARY KEY,
 Name          varchar(30) NOT NULL,
 ProductNumber nvarchar(25) NOT NULL,
 ListPrice     money NOT NULL,
 StandardCost  money NOT NULL)
GO

/*
   Daten
*/
-- Daten lschen
DELETE FROM Production.Product2
-- Datei eintragen
INSERT INTO Production.Product2
(Name, ProductNumber, ListPrice, StandardCost)
SELECT TOP 5 Name, ProductNumber, ListPrice, StandardCost
  FROM Production.Product
 WHERE ListPrice > 0
-- berprfung
SELECT * FROM Production.Product2

/*
   Prozeduren
*/ 
-- Prozedur, die XML erstellt
IF OBJECT_ID ('Production.GetProduct') IS NOT NULL
 DROP PROCEDURE Production.GetProduct
GO
CREATE PROCEDURE Production.GetProduct(@vProductNumber nvarchar(25))
AS BEGIN
 SELECT * FROM Production.Product2 AS Product
  WHERE ProductNumber = @vProductNumber
    FOR XML AUTO, ELEMENTS
END
-- Test
EXEC Production.GetProduct @vProductNumber = 'SA-M237'

-- Prozedur, die XML erwartet
IF OBJECT_ID ('Production.UpdateProduct') IS NOT NULL
 DROP PROCEDURE Production.UpdateProduct
GO
CREATE PROCEDURE Production.UpdateProduct(@vProductXML nvarchar(max))
AS BEGIN
 -- Variablen
 DECLARE @vProductID     int,
         @vName          nvarchar(30),
         @vProductNumber nvarchar(25),
         @vListPrice     money,
         @vStandardCost  money,
         @idoc int
 -- Standardzerlegung und gleichzeitige Eintragung
 EXEC sp_xml_preparedocument @idoc OUTPUT, @vProductXML 
 SELECT @vProductID     = ProductID,
        @vName          = Name,
        @vProductNumber = ProductNumber,
        @vListPrice     = ListPrice,
        @vStandardCost  = StandardCost
   FROM OPENXML(@idoc, '/Product',2)
   WITH (ProductID     int,
         Name          varchar(50),
         ProductNumber varchar(25),
         ListPrice     money,
         StandardCost  money)
 -- Aktualisierung
 UPDATE Production.Product2
    SET Name = @vName,
        ProductNumber = @vProductNumber,
        ListPrice = @vListPrice,
        StandardCost = @vStandardCost
  WHERE ProductID = @vProductID
END
-- berprfung
DECLARE @vProductXML nvarchar(max)
SET @vProductXML = '<Product>
  <ProductID>2</ProductID>
  <Name>ML Mountain Seat Assembly</Name>
  <ProductNumber>SA-M237</ProductNumber>
  <ListPrice>147.1400</ListPrice>
  <StandardCost>108.9900</StandardCost>
</Product>'
EXEC Production.UpdateProduct @vProductXML = @vProductXML
